package com.framsticks.core;

import com.framsticks.params.FramsClass;
import com.framsticks.params.Param;
import com.framsticks.params.types.DecimalParam;
import com.framsticks.params.types.EnumParam;
import com.framsticks.params.types.StringParam;
import com.framsticks.util.lang.Strings;

import java.util.Arrays;

/**
 * @author Piotr Sniegowski
 */
public class ListChange {

	public Action getAction() {
		return action;
	}

	public Integer getPosition() {
		return position;
	}

	public String getIdentifier() {
		return identifier;
	}

	public static enum Action {
		Add,
		Remove,
		Modify
	}

	private Action action;
	private Integer position;
	private String identifier;

	public Integer getType() { return action.ordinal(); }
	public void setType(Integer type) { action = Action.values()[type]; }

	public Integer getPos() { return position; }
	public void setPos(Integer pos) { position = pos; }

	public String getId() { return identifier; }
	public void setId(String id) { identifier = id; }

	public String getBestIdentifier() {
		if (Strings.notEmpty(identifier)) {
			return identifier;
		}
		return position.toString();
	}

	public static FramsClass getFramsClass() {
		return new FramsClass("ListChange", "ListChange", null)
				.append(Param.build().id("type").name("type").type(new EnumParam(Arrays.asList("Add", "Remove", "Modify"))))
				.append(Param.build().id("id").name("identifier").type(StringParam.class))
				.append(Param.build().id("pos").name("position").type(DecimalParam.class));
	}

	@Override
	public String toString() {
		return action + " " + identifier + " " + position;
	}
}
